@@ -23,9 +23,15 @@ def loadRecords(extractedDataPaths, options, recordIndices=None, sliceLength=sli
     return np.concatenate(arrayList, axis=2)
 
 
-
-
-def fillBatchQueue(extractedDataPaths, options, resultQueue=None, resultSize=30, batchLengths=None, startFromBatch=0, recordSlices=None):
+def fillBatchQueue(
+    extractedDataPaths,
+    options,
+    resultQueue=None,
+    resultSize=30,
+    batchLengths=None,
+    startFromBatch=0,
+    recordSlices=None,
+):
 
     fixedSize = options["shape"][1] if batchLengths is None else None
 
@@ -38,7 +44,6 @@ def fillBatchQueue(extractedDataPaths, options, resultQueue=None, resultSize=30,
             arr = flatten(records)
         return arr
 
-
     def getNumbersOfRecordsInBatch():
         return len(batchLengths) if fixedSize is None else options["shape"][0]
 
@@ -51,11 +56,10 @@ def fillBatchQueue(extractedDataPaths, options, resultQueue=None, resultSize=30,
 
     def recordPositionEnd(index):
         l = getBatchLengths()
-        return sum(l[0:(index+1)])
+        return sum(l[0 : (index + 1)])
 
     def getRecordsFromTo(start, stop=None):
-        """ get all available records from a slice
-        """
+        """get all available records from a slice"""
         if stop is None:
             stop = getNumbersOfRecordsInBatch()
 
@@ -112,7 +116,10 @@ def fillBatchQueue(extractedDataPaths, options, resultQueue=None, resultSize=30,
 
                 # iterate over all possible batches within the record slice
                 while recordsInBatch + recordsInSlice >= resultSize:
-                    currentSlice = slice(currentSlice.start, currentSlice.start + resultSize - recordsInBatch)
+                    currentSlice = slice(
+                        currentSlice.start,
+                        currentSlice.start + resultSize - recordsInBatch,
+                    )
                     if skipFirstBatches <= 0:
                         res += getRecordsFromSlice(currentSlice)
                         resultQueue.put(res)
@@ -134,12 +141,17 @@ def fillBatchQueue(extractedDataPaths, options, resultQueue=None, resultSize=30,
                     resultQueue.put(res)
                 skipFirstBatches -= 1
 
+
 class RecordNumpyMemmapExporter(DataExporter):
     """Asynchrone np exporter"""
 
     includesStorage = True
     batchFillProcess = None
 
+    def __init__(self, options=None):
+        super().__init__(options)
+        self.currentArrayShapes = {}
+
     def initialOptions(self):
         return {
             "basePath": "data/",
@@ -150,9 +162,10 @@ class RecordNumpyMemmapExporter(DataExporter):
             "combine": None,
             "recordSlices": None,
         }
-        
-    def getShapeFilePath(self, dataId):
-        return self.getPath(dataId) + "-shape.npy"
+
+    def getShapeFilePath(self, dataId, tmp=False):
+        add = "-tmp" if tmp else ""
+        return self.getPath(dataId) + add + "-shape.npy"
 
     def saveFromArrayList(self, dataId, arrayList):
         """fills the array from a list of 2-dimensional arrays"""
@@ -162,13 +175,46 @@ class RecordNumpyMemmapExporter(DataExporter):
         memmap[:, :, :] = array
         memmap.flush()
         del memmap
-        
-        lengths = [r.shape[0] for r in arrayList]        
-        shape = np.array([lengths, array.shape], dtype="object")
-        np.save(self.getShapeFilePath(dataId), shape)
-        self.log("saved %s: %s" % (dataId, str(shape)))
+
+        lengths = [r.shape[0] for r in arrayList]
+        self.currentArrayShapes[dataId] = lengths, array.shape
         self.finishStream(dataId)
 
+    def saveAndAppendArray(self, dataId, arrayList):
+        """appends an array to the existing file or creates a new one"""
+
+        array = np.concatenate(arrayList)
+        array = np.expand_dims(array, 0)
+
+        memmap = None
+        if dataId not in self.currentArrayShapes:
+            self.currentArrayShapes[dataId] = [[], np.array((1, 0, array.shape[2]))]
+            memmap = self.stream(dataId, {"dtype": "float32", "mode": "w+", "shape": array.shape})
+
+        shape = self.currentArrayShapes[dataId][1]
+        if array.shape[2] != shape[2]:
+            raise Exception("The number of channels has to be constant, but was %s and is now %s" % (shape[2], array.shape[2]))
+
+        lengths = shape[1]
+
+        shape[1] += array.shape[1]
+
+        if memmap is None:
+            memmap = self.stream(
+                dataId,
+                {
+                    "dtype": "float32",
+                    "mode": "readwrite",
+                    "shape": tuple(shape.tolist()),
+                },
+            )
+        memmap[:, lengths:, :] = array
+        memmap.flush()
+        del memmap
+
+        self.currentArrayShapes[dataId][0] += [r.shape[0] for r in arrayList]
+        self.currentArrayShapes[dataId][1] = shape
+        self.saveShape(dataId, *self.currentArrayShapes[dataId], tmp=True)
 
     def checkType(self, type):
         return type == np.memmap
@@ -179,7 +225,21 @@ class RecordNumpyMemmapExporter(DataExporter):
     def stream(self, dataId, options):
         return np.memmap(self.getPath(dataId + "-tmp"), **options)
 
+    def saveShape(self, dataId, lengths, saveShape, tmp=False):
+        shape = np.array((lengths, saveShape), dtype="object")
+        np.save(self.getShapeFilePath(dataId, tmp=tmp), shape)
+        self.log("saved %s: %s" % (dataId, str(shape)))
+
+    def finishShape(self, dataId):
+        self.saveShape(dataId, *self.currentArrayShapes[dataId], tmp=False)
+        del self.currentArrayShapes[dataId]
+        tmpPath = Path(self.getShapeFilePath(dataId, tmp=True))
+        if tmpPath.is_file():
+            tmpPath.unlink()
+
     def finishStream(self, dataId, options={}):
+        if dataId in self.currentArrayShapes:
+            self.finishShape(dataId)
         path = self.getPath(dataId)
         return shutil.move(path + "-tmp", path)
 
@@ -206,16 +266,15 @@ class RecordNumpyMemmapExporter(DataExporter):
                     lengths, shape = np.load(shapePath, allow_pickle=True)
                     if "memmap" not in options:
                         options["memmap"] = {}
-                        
+
                     options["lengths"] = lengths
                     options["memmap"]["shape"] = tuple(shape)
-        
+
         if "dtype" not in options["memmap"]:
             options["memmap"]["dtype"] = "float32"
-            
+
         if "mode" not in options["memmap"]:
             options["memmap"]["mode"] = "r"
-            
 
         self.memmapOptions = options["memmap"]
         self.extractedDataPaths = [(size, self.getPath(name)) for name, size in zip(names, sizes)]
